@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from typing import Any
 from typing import NoReturn
@@ -85,6 +75,8 @@ class ArgumentParser(argparse.ArgumentParser):
 class ChatCommands(Observable):
     def __init__(self) -> None:
         Observable.__init__(self)
+
+    def init(self) -> None:
         self._parser = ArgumentParser(prog='ChatCommands')
         self._sub_parser = self._parser.add_subparsers(title='Commands')
         self._commands: dict[str, tuple[list[str], str]] = {}
@@ -135,9 +127,6 @@ class ChatCommands(Observable):
         self._commands[command_name] = (used_in, usage)
 
     def parse(self, type_: str, arg_string: str) -> bool:
-        if arg_string.startswith('/me '):
-            return False
-
         if regex.COMMAND_REGEX.match(arg_string) is None:
             return False
 
@@ -164,6 +153,9 @@ class ChatCommands(Observable):
             self.notify('command-error', str(error))
             raise CommandFailed
 
+        if result == 'silent':
+            return True
+
         if result is None:
             result = _('Command executed successfully')
 
@@ -207,6 +199,12 @@ class ChatCommands(Observable):
                             choices=['moderator', 'participant', 'visitor'])
         self.add_command('role', ['groupchat'], parser)
 
+        parser = self.make_parser('me', self._me_command)
+        parser.add_argument('message', default=None, nargs='*')
+        self.add_command('me', ['chat', 'groupchat', 'pm'], parser)
+
+        app.plugin_manager.extension_point('add_commands', self)
+
     def _help_command(self, args: Any) -> str:
         return self._generate_help(args.used_in)
 
@@ -309,3 +307,7 @@ class ChatCommands(Observable):
 
     def _role_command(self, args: Any) -> None:
         self._change_role(args.who, args.role, None)
+
+    def _me_command(self, _args: Any) -> str:
+        app.window.activate_action('send-message', None)
+        return 'silent'
