@@ -13,28 +13,13 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
 from typing import Any
-from typing import Callable
-from typing import Optional
 from typing import TYPE_CHECKING
-from typing import Union
 
-import collections
-import copy
 import functools
 import hashlib
 import importlib.metadata
@@ -49,11 +34,14 @@ import socket
 import string
 import sys
 import unicodedata
+import uuid
 import weakref
 import webbrowser
 from collections import defaultdict
+from collections.abc import Callable
 from datetime import datetime
 from datetime import timedelta
+from datetime import timezone
 from functools import wraps
 from pathlib import Path
 from string import Template
@@ -61,11 +49,14 @@ from urllib.parse import unquote
 from urllib.parse import urlparse
 
 import precis_i18n.codec  # noqa: F401
+import qrcode
 from cryptography import x509
 from cryptography.hazmat.backends import default_backend
+from gi.repository import GdkPixbuf
 from gi.repository import Gio
 from gi.repository import GLib
 from gi.repository import GObject
+from gi.repository import Soup
 from nbxmpp.const import Affiliation
 from nbxmpp.const import Chatstate
 from nbxmpp.const import ConnectionProtocol
@@ -79,7 +70,9 @@ from nbxmpp.protocol import JID
 from nbxmpp.structs import CommonError
 from nbxmpp.structs import ProxyData
 from packaging.requirements import Requirement
+from packaging.specifiers import SpecifierSet
 from packaging.version import Version as V
+from qrcode.image.pil import PilImage as QrcPilImage
 
 from gajim.common import app
 from gajim.common import configpaths
@@ -118,6 +111,7 @@ log = logging.getLogger('gajim.c.helpers')
 
 URL_REGEX = re.compile(
     r"(www\.(?!\.)|[a-z][a-z0-9+.-]*://)[^\s<>'\"]+[^!,\.\s<>\)'\"\]]")
+CURRENT_PYTHON_VERSION = platform.python_version()
 
 
 class InvalidFormat(Exception):
@@ -160,7 +154,7 @@ def ascii_to_idn(host: str) -> str:
     return '.'.join(converted_labels)
 
 
-def parse_resource(resource: str) -> Optional[str]:
+def parse_resource(resource: str) -> str | None:
     '''
     Perform stringprep on resource and return it
     '''
@@ -195,7 +189,7 @@ def get_uf_sub(sub: str) -> str:
     return p_('Contact subscription', 'Unknown')
 
 
-def get_uf_ask(ask: Union[str, None]) -> str:
+def get_uf_ask(ask: str | None) -> str:
     if ask is None:
         return p_('Contact subscription', 'None')
 
@@ -205,7 +199,7 @@ def get_uf_ask(ask: Union[str, None]) -> str:
     return ask
 
 
-def get_uf_role(role: Union[Role, str], plural: bool = False) -> str:
+def get_uf_role(role: Role | str, plural: bool = False) -> str:
     ''' plural determines if you get Moderators or Moderator'''
     if not isinstance(role, str):
         role = role.value
@@ -227,7 +221,7 @@ def get_uf_role(role: Union[Role, str], plural: bool = False) -> str:
     return ''
 
 
-def get_uf_affiliation(affiliation: Union[Affiliation, str],
+def get_uf_affiliation(affiliation: Affiliation | str,
                        plural: bool = False
                        ) -> str:
     '''Get a nice and translated affilition for muc'''
@@ -252,7 +246,7 @@ def get_uf_affiliation(affiliation: Union[Affiliation, str],
 
 
 def get_uf_relative_time(date_time: datetime,
-                         now: Optional[datetime] = None
+                         now: datetime | None = None
                          ) -> str:
 
     if now is None:  # used by unittest
@@ -297,7 +291,7 @@ def from_one_line(msg: str) -> str:
     return msg.replace('\\\\', '\\')
 
 
-def chatstate_to_string(chatstate: Optional[Chatstate]) -> str:
+def chatstate_to_string(chatstate: Chatstate | None) -> str:
     if chatstate is None:
         return ''
 
@@ -366,8 +360,23 @@ def get_contact_dict_for_account(account: str) -> dict[str, types.BareContact]:
     return contacts_dict
 
 
+def generate_qr_code(content: str) -> GdkPixbuf.Pixbuf | None:
+    qr = qrcode.QRCode(version=None,
+                       error_correction=qrcode.constants.ERROR_CORRECT_L,
+                       box_size=6,
+                       border=4)
+    qr.add_data(content)
+    qr.make(fit=True)
+
+    img = qr.make_image(image_factory=QrcPilImage).convert('RGB')
+    return GdkPixbuf.Pixbuf.new_from_bytes(
+        GLib.Bytes.new(img.tobytes()),
+        GdkPixbuf.Colorspace.RGB, False, 8,
+        img.width, img.height, img.width*3)
+
+
 def play_sound(sound_event: str,
-               account: Optional[str] = None,
+               account: str | None = None,
                force: bool = False,
                loop: bool = False) -> None:
 
@@ -380,8 +389,8 @@ def play_sound(sound_event: str,
 
 
 def check_soundfile_path(file_: str,
-                         dirs: Optional[list[Path]] = None
-                         ) -> Optional[Path]:
+                         dirs: list[Path] | None = None
+                         ) -> Path | None:
     '''
     Check if the sound file exists
 
@@ -406,8 +415,8 @@ def check_soundfile_path(file_: str,
     return None
 
 
-def strip_soundfile_path(file_: Union[Path, str],
-                         dirs: Optional[list[Path]] = None,
+def strip_soundfile_path(file_: Path | str,
+                         dirs: list[Path] | None = None,
                          abs_: bool = True):
     '''
     Remove knowns paths from a sound file
@@ -507,7 +516,7 @@ def statuses_unified() -> bool:
     return True
 
 
-def get_full_jid_from_iq(iq_obj: Iq) -> Optional[str]:
+def get_full_jid_from_iq(iq_obj: Iq) -> str | None:
     '''
     Return the full jid (with resource) from an iq
     '''
@@ -517,7 +526,7 @@ def get_full_jid_from_iq(iq_obj: Iq) -> Optional[str]:
     return parse_jid(str(iq_obj.getFrom()))
 
 
-def get_jid_from_iq(iq_obj: Iq) -> Optional[str]:
+def get_jid_from_iq(iq_obj: Iq) -> str | None:
     '''
     Return the jid (without resource) from an iq
     '''
@@ -657,7 +666,7 @@ def jid_is_blocked(account: str, jid: str) -> bool:
     return jid in client.get_module('Blocking').blocked
 
 
-def get_subscription_request_msg(account: Optional[str] = None) -> str:
+def get_subscription_request_msg(account: str | None = None) -> str:
     if account is None:
         return _('I would like to add you to my contact list.')
 
@@ -670,27 +679,26 @@ def get_subscription_request_msg(account: Optional[str] = None) -> str:
     return Template(message).safe_substitute({'name': app.nicks[account]})
 
 
-def get_retraction_text(account: str,
-                        moderator_jid: str,
-                        reason: Optional[str]) -> str:
+def get_moderation_text(by: str | JID | None,
+                        reason: str | None) -> str:
 
-    client = app.get_client(account)
-    contact = client.get_module('Contacts').get_contact(
-        moderator_jid, groupchat=True)
-    text = _('This message has been retracted by %s.') % contact.name
+    by_text = ''
+    if by is not None:
+        by_text = (' by %s') % by
+    text = _('This message has been moderated%s.') % by_text
     if reason is not None:
         text += ' ' + _('Reason: %s') % reason
     return text
 
 
-def get_global_proxy() -> Optional[ProxyData]:
+def get_global_proxy() -> ProxyData | None:
     proxy_name = app.settings.get('global_proxy')
     if not proxy_name:
         return None
     return get_proxy(proxy_name)
 
 
-def get_account_proxy(account: str, fallback=True) -> Optional[ProxyData]:
+def get_account_proxy(account: str, fallback=True) -> ProxyData | None:
     proxy_name = app.settings.get_account_setting(account, 'proxy')
     if proxy_name:
         return get_proxy(proxy_name)
@@ -700,7 +708,7 @@ def get_account_proxy(account: str, fallback=True) -> Optional[ProxyData]:
     return None
 
 
-def get_proxy(proxy_name: str) -> Optional[ProxyData]:
+def get_proxy(proxy_name: str) -> ProxyData | None:
     try:
         settings = app.settings.get_proxy_settings(proxy_name)
     except ValueError:
@@ -716,7 +724,7 @@ def get_proxy(proxy_name: str) -> Optional[ProxyData]:
                      password=password)
 
 
-def determine_proxy() -> Optional[ProxyData]:
+def determine_proxy() -> ProxyData | None:
     # Use this method to find a proxy for non-account related http requests
     # When there is no global proxy and at least one active account does
     # not use a proxy, we assume no proxy is necessary.
@@ -752,8 +760,8 @@ def call_counter(func):
 
 
 def load_json(path: Path,
-              key: Optional[str] = None,
-              default: Optional[Any] = None) -> Any:
+              key: str | None = None,
+              default: Any | None = None) -> Any:
     try:
         with path.open('r', encoding='utf8') as file:
             json_dict = json.loads(file.read())
@@ -766,60 +774,6 @@ def load_json(path: Path,
     return json_dict.get(key, default)
 
 
-class AdditionalDataDict(collections.UserDict):
-    data: dict[str, Any]
-
-    @staticmethod
-    def _get_path_childs(full_path: str) -> list[str]:
-        if ':' in full_path:
-            return full_path.split(':')
-        return [full_path]
-
-    def set_value(self, full_path: str, key: str, value: Optional[str]) -> None:
-        path_childs = self._get_path_childs(full_path)
-        _dict = self.data
-        for path in path_childs:
-            try:
-                _dict = _dict[path]
-            except KeyError:
-                _dict[path] = {}
-                _dict = _dict[path]
-        _dict[key] = value
-
-    def get_value(self,
-                  full_path: str,
-                  key: str,
-                  default: Optional[str] = None) -> Optional[Any]:
-
-        path_childs = self._get_path_childs(full_path)
-        _dict = self.data
-        for path in path_childs:
-            try:
-                _dict = _dict[path]
-            except KeyError:
-                return default
-        try:
-            return _dict[key]
-        except KeyError:
-            return default
-
-    def remove_value(self, full_path: str, key: str) -> None:
-        path_childs = self._get_path_childs(full_path)
-        _dict = self.data
-        for path in path_childs:
-            try:
-                _dict = _dict[path]
-            except KeyError:
-                return
-        try:
-            del _dict[key]
-        except KeyError:
-            return
-
-    def copy(self) -> AdditionalDataDict:
-        return copy.deepcopy(self)
-
-
 def delay_execution(milliseconds):
     # Delay the first call for `milliseconds`
     # ignore all other calls while the delay is active
@@ -1021,7 +975,7 @@ _xmpp_query_type_handlers = {
 
 
 @catch_exceptions
-def open_uri(uri: Union[URI, str], account: Optional[str] = None) -> None:
+def open_uri(uri: URI | str, account: str | None = None) -> None:
     if not isinstance(uri, URI):
         uri = parse_uri(uri)
 
@@ -1090,7 +1044,7 @@ def open_file_uri(uri: str) -> None:
         if sys.platform != 'win32':
             Gio.AppInfo.launch_default_for_uri(uri)
         else:
-            os.startfile(uri)
+            os.startfile(uri, 'open')  # noqa: S606
     except Exception as err:
         log.info("Couldn't open file URI %s: %s", uri, err)
 
@@ -1114,7 +1068,7 @@ def show_in_folder(path: Path) -> None:
         open_directory(path.parent)
 
 
-def filesystem_path_from_uri(uri: str) -> Optional[Path]:
+def filesystem_path_from_uri(uri: str) -> Path | None:
     puri = parse_uri(uri)
     if puri.type != URIType.FILE:
         return None
@@ -1125,7 +1079,7 @@ def filesystem_path_from_uri(uri: str) -> Optional[Path]:
     return Path(puri.data['path'])
 
 
-def get_file_path_from_dnd_dropped_uri(text: str) -> Optional[Path]:
+def get_file_path_from_dnd_dropped_uri(text: str) -> Path | None:
     uri = text.strip('\r\n\x00')  # remove \r\n and NULL
     return filesystem_path_from_uri(uri)
 
@@ -1165,7 +1119,7 @@ def geo_provider_from_location(lat: str, lon: str) -> str:
     return f'https://www.openstreetmap.org/?mlat={lat}&mlon={lon}&zoom=16'
 
 
-def get_resource(account: str) -> Optional[str]:
+def get_resource(account: str) -> str | None:
     resource = app.settings.get_account_setting(account, 'resource')
     if not resource:
         return None
@@ -1177,16 +1131,21 @@ def get_resource(account: str) -> Optional[str]:
     return resource
 
 
-def get_default_muc_config() -> dict[str, Union[bool, str]]:
+def get_default_muc_config() -> dict[str, bool | str]:
     return {
         # XEP-0045 options
+        # https://xmpp.org/registrar/formtypes.html
+
         'muc#roomconfig_allowinvites': True,
-        'muc#roomconfig_publicroom': False,
+        'muc#roomconfig_allowpm': 'anyone',
+        'muc#roomconfig_changesubject': False,
+        'muc#roomconfig_enablelogging': False,
         'muc#roomconfig_membersonly': True,
-        'muc#roomconfig_persistentroom': True,
-        'muc#roomconfig_whois': 'anyone',
         'muc#roomconfig_moderatedroom': False,
-        'muc#roomconfig_enablelogging': True,
+        'muc#roomconfig_passwordprotectedroom': False,
+        'muc#roomconfig_persistentroom': True,
+        'muc#roomconfig_publicroom': False,
+        'muc#roomconfig_whois': 'moderators',
 
         # Ejabberd options
         'allow_voice_requests': False,
@@ -1194,12 +1153,12 @@ def get_default_muc_config() -> dict[str, Union[bool, str]]:
         'mam': True,
 
         # Prosody options
-        '{http://prosody.im/protocol/muc}roomconfig_allowmemberinvites': True,
+        '{http://prosody.im/protocol/muc}roomconfig_allowmemberinvites': False,
         'muc#roomconfig_enablearchiving': True,
     }
 
 
-def validate_jid(jid: Union[str, JID], type_: Optional[str] = None) -> JID:
+def validate_jid(jid: str | JID, type_: str | None = None) -> JID:
     try:
         jid = JID.from_string(str(jid))
     except InvalidJid as error:
@@ -1217,7 +1176,7 @@ def validate_jid(jid: Union[str, JID], type_: Optional[str] = None) -> JID:
     raise ValueError(f'Not a {type_} JID')
 
 
-def to_user_string(error: Union[CommonError, StanzaError]) -> str:
+def to_user_string(error: CommonError | StanzaError) -> str:
     text = error.get_text(get_rfc5646_lang())
     if text:
         return text
@@ -1243,7 +1202,7 @@ def get_groupchat_name(client: types.Client, jid: JID) -> str:
 
 def is_affiliation_change_allowed(self_contact: types.GroupchatParticipant,
                                   contact: types.GroupchatParticipant,
-                                  target_aff: Union[str, Affiliation]) -> bool:
+                                  target_aff: str | Affiliation) -> bool:
     if isinstance(target_aff, str):
         target_aff = Affiliation(target_aff)
 
@@ -1272,7 +1231,7 @@ def is_role_change_allowed(self_contact: types.GroupchatParticipant,
     return self_contact.affiliation >= contact.affiliation
 
 
-def is_retraction_allowed(self_contact: types.GroupchatParticipant,
+def is_moderation_allowed(self_contact: types.GroupchatParticipant,
                           contact: types.GroupchatParticipant) -> bool:
 
     if self_contact.role < Role.MODERATOR:
@@ -1286,13 +1245,13 @@ def get_tls_error_phrases(tls_errors: set[Gio.TlsCertificateFlags]
 
 
 class Observable:
-    def __init__(self, log_: Union[logging.Logger, LogAdapter, None] = None):
+    def __init__(self, log_: logging.Logger | LogAdapter | None = None):
         self._log = log_
         self._callbacks: types.ObservableCbDict = defaultdict(list)
 
     def __disconnect(self,
                      obj: Any,
-                     signals: Optional[set[str]] = None
+                     signals: set[str] | None = None
                      ) -> None:
 
         def _remove(handlers: list[weakref.WeakMethod[types.AnyCallableT]]
@@ -1300,7 +1259,12 @@ class Observable:
 
             for handler in list(handlers):
                 func = handler()
-                if func is None or func.__self__ is obj:
+                # Don’t remove dead weakrefs from the handler list
+                # notify() will remove dead refs, and __disconnect()
+                # can be called from inside notify(), this can lead
+                # to race conditions where later notfiy tries to remove
+                # a dead ref which is not anymore in the list.
+                if func is not None and func.__self__ is obj:
                     handlers.remove(handler)
 
         if signals is None:
@@ -1316,7 +1280,7 @@ class Observable:
 
     def multi_disconnect(self,
                          obj: Any,
-                         signals: Optional[set[str]]
+                         signals: set[str] | None
                          ) -> None:
 
         self.__disconnect(obj, signals)
@@ -1338,6 +1302,10 @@ class Observable:
 
         weak_func = weakref.WeakMethod(func)
 
+        if weak_func in self._callbacks[signal_name]:
+            # Don’t register handler multiple times
+            return
+
         self._callbacks[signal_name].append(weak_func)
 
     def connect(self,
@@ -1368,8 +1336,8 @@ class Observable:
 def write_file_async(
         path: Path,
         data: bytes,
-        callback: Callable[[bool, Optional[GLib.Error], Any], Any],
-        user_data: Optional[Any] = None):
+        callback: Callable[[bool, GLib.Error | None, Any], Any],
+        user_data: Any | None = None):
 
     def _on_write_finished(outputstream: Gio.OutputStream,
                            result: Gio.AsyncResult,
@@ -1405,10 +1373,10 @@ def write_file_async(
 
 
 def load_file_async(path: Path,
-                    callback: Callable[[Optional[bytes],
-                                        Optional[GLib.Error],
-                                        Optional[Any]], Any],
-                    user_data: Optional[Any] = None) -> None:
+                    callback: Callable[[bytes | None,
+                                        GLib.Error | None,
+                                        Any | None], Any],
+                    user_data: Any | None = None) -> None:
 
     def _on_load_finished(file: Gio.File,
                           result: Gio.AsyncResult) -> None:
@@ -1430,16 +1398,17 @@ def get_x509_cert_from_gio_cert(cert: Gio.TlsCertificate) -> x509.Certificate:
         glib_bytes.get_data(), default_backend())
 
 
-def get_custom_host(account: str) -> Optional[tuple[str,
-                                                    ConnectionProtocol,
-                                                    ConnectionType]]:
+def get_custom_host(
+    account: str
+) -> tuple[str, ConnectionProtocol, ConnectionType] | None:
+
     if not app.settings.get_account_setting(account, 'use_custom_host'):
         return None
     host = app.settings.get_account_setting(account, 'custom_host')
     port = app.settings.get_account_setting(account, 'custom_port')
     type_ = app.settings.get_account_setting(account, 'custom_type')
 
-    if host.startswith('ws://') or host.startswith('wss://'):
+    if host.startswith(('ws://', 'wss://')):
         protocol = ConnectionProtocol.WEBSOCKET
     else:
         host = f'{host}:{port}'
@@ -1469,7 +1438,7 @@ def get_idle_status_message(state: str, status_message: str) -> str:
     }
 
 
-def get_group_chat_nick(account: str, room_jid: Union[JID, str]) -> str:
+def get_group_chat_nick(account: str, room_jid: JID | str) -> str:
     client = app.get_client(account)
 
     bookmark = client.get_module('Bookmarks').get_bookmark(room_jid)
@@ -1514,7 +1483,7 @@ def get_os_name() -> str:
 
 def get_os_version() -> str:
     if sys.platform in ('win32', 'darwin'):
-        return platform.release()
+        return platform.version()
     if os.name == 'posix':
         try:
             import distro
@@ -1534,6 +1503,12 @@ def get_glib_version() -> str:
                               GLib.MICRO_VERSION]))
 
 
+def get_soup_version() -> str:
+    return '.'.join(map(str, [Soup.get_major_version(),
+                              Soup.get_minor_version(),
+                              Soup.get_micro_version()]))
+
+
 def package_version(requirement: str) -> bool:
     req = Requirement(requirement)
 
@@ -1545,6 +1520,11 @@ def package_version(requirement: str) -> bool:
     return installed_version in req.specifier
 
 
+def python_version(specifier_set: str) -> bool:
+    spec = SpecifierSet(specifier_set)
+    return CURRENT_PYTHON_VERSION in spec
+
+
 def make_path_from_jid(base_path: Path, jid: JID) -> Path:
     assert jid.domain is not None
     domain = jid.domain[:50]
@@ -1556,3 +1536,24 @@ def make_path_from_jid(base_path: Path, jid: JID) -> Path:
     if jid.resource is not None:
         return path / jid.resource[:30]
     return path
+
+
+def format_idle_time(idle_time: datetime) -> str:
+    now = datetime.now(timezone.utc)
+
+    now_date = now.date()
+    idle_date = idle_time.date()
+
+    if idle_date == now_date:
+        return idle_time.strftime(app.settings.get('time_format'))
+    if idle_date == now_date - timedelta(days=1):
+        return _('Yesterday (%s)') % idle_time.strftime(
+            app.settings.get('time_format'))
+    if idle_date >= now_date - timedelta(days=6):
+        return idle_time.strftime(f'%a {app.settings.get("time_format")}')
+
+    return idle_date.strftime(app.settings.get('date_format'))
+
+
+def get_uuid() -> str:
+    return str(uuid.uuid4())
