@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 # XEP-0045: Multi-User Chat
 # XEP-0249: Direct MUC Invitations
@@ -47,7 +37,6 @@ from gajim.common import helpers
 from gajim.common import types
 from gajim.common.const import ClientState
 from gajim.common.const import MUCJoinedState
-from gajim.common.events import MessageModerated
 from gajim.common.events import MucAdded
 from gajim.common.events import MucDecline
 from gajim.common.events import MucInvitation
@@ -59,6 +48,7 @@ from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.structs import MUCData
 from gajim.common.structs import MUCPresenceData
+from gajim.common.util.datetime import utc_now
 
 log = logging.getLogger('gajim.c.m.muc')
 
@@ -82,7 +72,7 @@ class MUC(BaseModule):
         'approve_voice_request',
         'destroy',
         'request_disco_info',
-        'retract_message'
+        'moderate_message'
     ]
 
     def __init__(self, con: types.Client) -> None:
@@ -99,6 +89,11 @@ class MUC(BaseModule):
                           typ='unavailable',
                           ns=Namespace.MUC_USER,
                           priority=49),
+            StanzaHandler(name='message',
+                          callback=self._on_muc_normal_user_message,
+                          typ='normal',
+                          ns=Namespace.MUC_USER,
+                          priority=49),
             StanzaHandler(name='presence',
                           callback=self._on_error_presence,
                           typ='error',
@@ -107,11 +102,6 @@ class MUC(BaseModule):
                           callback=self._on_subject_change,
                           typ='groupchat',
                           priority=49),
-            StanzaHandler(name='message',
-                          callback=self._on_moderation,
-                          ns=Namespace.FASTEN,
-                          typ='groupchat',
-                          priority=49),
             StanzaHandler(name='message',
                           callback=self._on_config_change,
                           ns=Namespace.MUC_USER,
@@ -238,7 +228,7 @@ class MUC(BaseModule):
             if bookmark.password is not None:
                 password = bookmark.password
 
-        return MUCData(room_jid, nick, password, config)
+        return MUCData(room_jid, nick, None, password, config)
 
     def join(self,
              jid: JID,
@@ -460,7 +450,8 @@ class MUC(BaseModule):
         self._log.info('Configuration finished: %s', jid)
 
         room = self._get_contact(jid.bare)
-        event = events.MUCRoomConfigFinished(timestamp=time.time())
+        event = events.MUCRoomConfigFinished(
+            timestamp=utc_now())
 
         assert isinstance(room, GroupchatContact)
         app.storage.events.store(room, event)
@@ -546,7 +537,7 @@ class MUC(BaseModule):
         else:
             error = helpers.to_user_string(properties.error)
             event = events.MUCRoomPresenceError(
-                timestamp=time.time(),
+                timestamp=utc_now(),
                 error=error)
             assert isinstance(room, GroupchatContact)
             app.storage.events.store(room, event)
@@ -568,6 +559,8 @@ class MUC(BaseModule):
         occupant = self._get_contact(properties.jid, groupchat=True)
         room = self._get_contact(properties.jid.bare)
 
+        timestamp = utc_now()
+
         if properties.is_muc_destroyed:
             self._log.info('MUC destroyed: %s', room_jid)
             self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
@@ -575,7 +568,7 @@ class MUC(BaseModule):
             room.set_not_joined()
 
             event = events.MUCRoomDestroyed(
-                timestamp=time.time(),
+                timestamp=timestamp,
                 reason=properties.muc_destroyed.reason,
                 alternate=properties.muc_destroyed.alternate)
             assert isinstance(room, GroupchatContact)
@@ -610,7 +603,7 @@ class MUC(BaseModule):
                                                    occupant)
 
             event = events.MUCNicknameChanged(
-                timestamp=time.time(),
+                timestamp=timestamp,
                 is_self=properties.is_muc_self_presence,
                 new_name=new_occupant.name,
                 old_name=occupant.name)
@@ -626,6 +619,9 @@ class MUC(BaseModule):
             if properties.is_muc_self_presence:
                 self._log.info('Self presence: %s', properties.jid)
                 if muc_data.state == MUCJoinedState.JOINING:
+                    if room.supports(Namespace.OCCUPANT_ID):
+                        muc_data.occupant_id = properties.occupant_id
+
                     if (properties.is_nickname_modified or
                             muc_data.nick != properties.muc_nickname):
                         muc_data.nick = properties.muc_nickname
@@ -634,6 +630,7 @@ class MUC(BaseModule):
 
                 elif muc_data.state == MUCJoinedState.CREATING:
                     if properties.is_new_room:
+                        muc_data.occupant_id = properties.occupant_id
                         self.configure_room(room_jid)
 
             presence = self._process_user_presence(properties)
@@ -647,7 +644,7 @@ class MUC(BaseModule):
             room.set_not_joined()
 
             event = events.MUCRoomKicked(
-                timestamp=time.time(),
+                timestamp=timestamp,
                 status_codes=properties.muc_status_codes,
                 reason=properties.muc_user.reason,
                 actor=properties.muc_user.actor)
@@ -668,6 +665,11 @@ class MUC(BaseModule):
         try:
             presence = self._process_user_presence(properties)
         except KeyError:
+            if (properties.type.is_unavailable
+                    and properties.muc_user.role.is_none):
+                # prosody allows broadcasting "unavailable" presences from
+                # offline room members
+                return
             # Sometimes it seems to happen that we get unavailable presence
             # from occupants we don’t know
             log.warning('Unexpected presence received')
@@ -682,7 +684,7 @@ class MUC(BaseModule):
             presence: MUCPresenceData,
             occupant: GroupchatParticipant) -> None:
 
-        timestamp = time.time()
+        timestamp = utc_now()
 
         if not occupant.is_available and presence.available:
 
@@ -761,6 +763,38 @@ class MUC(BaseModule):
         for signal, event in signals_and_events:
             occupant.notify(signal, event)
 
+    def _on_muc_normal_user_message(self,
+                                    _con: types.xmppClient,
+                                    stanza: Presence,
+                                    properties: MessageProperties) -> None:
+        if not properties.muc_user or not properties.muc_user.jid:
+            return
+
+        if not properties.muc_user.affiliation and not properties.muc_user.role:
+            return
+
+        room_jid = str(properties.muc_jid)
+        if room_jid not in self._mucs:
+            self._log.warning('Message from unknown MUC')
+            self._log.warning(stanza)
+            return
+
+        room = self._get_contact(room_jid)
+
+        if properties.muc_user.nick is not None:
+            nick = properties.muc_user.nick
+        else:
+            contact = self._get_contact(properties.muc_user.jid)
+            nick = contact.name
+
+        event = events.MUCAffiliationChanged(
+            timestamp=utc_now(),
+            nick=nick,
+            affiliation=properties.muc_user.affiliation,
+        )
+        app.storage.events.store(room, event)
+        room.notify('room-affiliation-changed', event)
+
     def _process_user_presence(self,
                                properties: PresenceProperties
                                ) -> MUCPresenceData:
@@ -847,24 +881,6 @@ class MUC(BaseModule):
 
         raise nbxmpp.NodeProcessed
 
-    def _on_moderation(self,
-                       _con: types.xmppClient,
-                       _stanza: Message,
-                       properties: MessageProperties
-                       ) -> None:
-        if not properties.is_moderation:
-            return
-
-        app.storage.archive.update_additional_data(
-            self._account, properties.moderation.stanza_id, properties)
-
-        app.ged.raise_event(
-            MessageModerated(account=self._account,
-                             jid=properties.jid,
-                             moderation=properties.moderation))
-
-        raise nbxmpp.NodeProcessed
-
     def cancel_password_request(self, room_jid: JID) -> None:
         self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
 
@@ -1010,7 +1026,7 @@ class MUC(BaseModule):
 
         assert properties.muc_status_codes is not None
         event = events.MUCRoomConfigChanged(
-            timestamp=time.time(),
+            timestamp=utc_now(),
             status_codes=properties.muc_status_codes)
 
         assert properties.muc_jid is not None
