@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -18,7 +8,7 @@ from typing import Any
 from typing import NamedTuple
 from typing import TypeVar
 
-import time
+import dataclasses
 from dataclasses import dataclass
 from dataclasses import fields
 from datetime import datetime
@@ -30,15 +20,18 @@ from nbxmpp.const import Role
 from nbxmpp.modules.dataforms import SimpleDataForm
 from nbxmpp.modules.security_labels import SecurityLabel
 from nbxmpp.protocol import JID
+from nbxmpp.structs import EncryptionData
 from nbxmpp.structs import MucSubject
 from nbxmpp.structs import PresenceProperties
+from nbxmpp.util import generate_id
 
 from gajim.common import types
-from gajim.common.const import KindConstant
 from gajim.common.const import MUCJoinedState
 from gajim.common.const import PresenceShowExt
 from gajim.common.const import URIType
+from gajim.common.storage.archive.const import MessageType
 from gajim.common.util.datetime import convert_epoch_to_local_datetime
+from gajim.common.util.datetime import utc_now
 
 _T = TypeVar('_T')
 
@@ -55,6 +48,7 @@ class MUCData:
     def __init__(self,
                  room_jid: str,
                  nick: str,
+                 occupant_id: str | None,
                  password: str | None,
                  config: dict[str, Any] | None = None
                  ) -> None:
@@ -63,6 +57,7 @@ class MUCData:
         self._config = config
         self.nick = nick
         self.password = password
+        self.occupant_id = occupant_id
         self.state = MUCJoinedState.NOT_JOINED
         # Message id of the captcha challenge
         self.captcha_id: str | None = None
@@ -85,124 +80,71 @@ class MUCData:
         return self._config
 
 
+@dataclass(slots=True)
 class OutgoingMessage:
-    def __init__(self,
-                 account: str,
-                 contact: types.ChatContactT,
-                 message: str | None,
-                 type_: str,
-                 subject: str | None = None,
-                 chatstate: str | None = None,
-                 marker: tuple[str, str] | None = None,
-                 resource: str | None = None,
-                 user_nick: str | None = None,
-                 label: SecurityLabel | None = None,
-                 control: Any | None = None,
-                 attention: bool | None = None,
-                 correct_id: str | None = None,
-                 oob_url: str | None = None,
-                 xhtml: str | None = None,
-                 nodes: Any | None = None,
-                 play_sound: bool = True
-                 ) -> None:
+    account: str
+    contact: types.ChatContactT
+    text: dataclasses.InitVar[str | None] = None
+    chatstate: str | None = None
+    marker: tuple[str, str] | None = None
+    sec_label: SecurityLabel | None = None
+    control: Any | None = None
+    correct_id: str | None = None
+    reply_data: ReplyData | None = None
+    reaction_data: tuple[str, set[str]] | None = None
+    oob_url: str | None = None
+    play_sound: bool = True
 
-        if type_ not in ('chat', 'groupchat', 'normal', 'headline'):
-            raise ValueError(f'Unknown message type: {type_}')
+    type: MessageType = dataclasses.field(init=False)
+    message_id: str = dataclasses.field(init=False)
+    timestamp: datetime = dataclasses.field(init=False)
 
-        if not message and chatstate is None and marker is None:
-            raise ValueError('Trying to send message without content')
+    _text: str | None = dataclasses.field(init=False)
+    _encryption_data: EncryptionData | None = dataclasses.field(
+        init=False, default=None)
+    _stanza: Any = dataclasses.field(init=False, default=None)
 
-        self.account = account
-        self.contact = contact
-        self.message = message
-        self.type_ = type_
-
-        if type_ == 'chat':
-            self.kind = KindConstant.CHAT_MSG_SENT
-        elif type_ == 'groupchat':
-            self.kind = KindConstant.GC_MSG
-        elif type_ == 'normal':
-            self.kind = KindConstant.SINGLE_MSG_SENT
-        else:
-            raise ValueError('Unknown message type')
-
-        from gajim.common.helpers import AdditionalDataDict
-        self.additional_data = AdditionalDataDict()
-
-        self.subject = subject
-        self.chatstate = chatstate
-        self.marker = marker
-        self.resource = resource
-        self.user_nick = user_nick
-        self.label = label
-        self.control = control
-        self.attention = attention
-        self.correct_id = correct_id
-
-        self.oob_url = oob_url
-
-        if oob_url is not None:
-            self.additional_data.set_value('gajim', 'oob_url', oob_url)
-
-        self.xhtml = xhtml
-
-        if xhtml is not None:
-            self.additional_data.set_value('gajim', 'xhtml', xhtml)
-
-        self.nodes = nodes
-        self.play_sound = play_sound
-
-        self.timestamp = None
-        self.message_id = None
-        self.stanza = None
-        self.delayed = None  # TODO never set
-
-        self.is_loggable = True
-
-    def copy(self):
-        message = OutgoingMessage(self.account,
-                                  self.contact,
-                                  self.message,
-                                  self.type_)
-        for name, value in vars(self).items():
-            setattr(message, name, value)
-        message.additional_data = self.additional_data.copy()
-        return message
-
-    @property
-    def jid(self) -> JID:
-        return self.contact.jid
+    def __post_init__(self, text: str | None) -> None:
+        self._text = text
+        self.timestamp = utc_now()
+        self.message_id = generate_id()
+        self.type = MessageType.from_str(self.contact.type_string)
 
     @property
     def is_groupchat(self) -> bool:
-        return self.type_ == 'groupchat'
+        return self.type == MessageType.GROUPCHAT
 
     @property
-    def is_chat(self) -> bool:
-        return self.type_ == 'chat'
+    def is_pm(self) -> bool:
+        return self.type == MessageType.PM
 
-    @property
-    def is_normal(self) -> bool:
-        return self.type_ == 'normal'
+    def get_text(self, with_fallback: bool = True) -> str | None:
+        if not with_fallback:
+            return self._text
 
-    def set_sent_timestamp(self) -> None:
-        if self.is_groupchat:
-            return
-        self.timestamp = time.time()
+        if self.reply_data is None:
+            return self._text
+        assert self._text is not None
+        return f'{self.reply_data.fallback_text}{self._text}'
+
+    def has_text(self) -> bool:
+        return bool(self._text)
+
+    def set_encryption(self, data: EncryptionData) -> None:
+        self._encryption_data = data
+
+    def get_encryption(self) -> EncryptionData | None:
+        return self._encryption_data
 
     @property
     def is_encrypted(self) -> bool:
-        return bool(self.additional_data.get_value('encrypted', 'name', False))
+        return self._encryption_data is not None
 
-    @property
-    def msg_iq(self) -> Any:
-        # Backwards compatibility for plugins
-        return self.stanza
+    def set_stanza(self, stanza: Any) -> None:
+        self._stanza = stanza
 
-    @msg_iq.setter
-    def msg_iq(self, value: Any) -> None:
-        # Backwards compatibility for plugins
-        self.stanza = value
+    def get_stanza(self) -> Any:
+        return self._stanza
 
 
 @dataclass(frozen=True)
@@ -342,3 +284,13 @@ class VariantMixin:
                     raise ValueError(f'no conversion for: {value_t_name}')
                 vdict[field_name] = conversion_func(value)
         return cls(**vdict)
+
+
+@dataclass
+class ReplyData:
+    pk: int
+    to: JID
+    id: str
+    fallback_start: int
+    fallback_end: int
+    fallback_text: str
