@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -30,22 +20,24 @@ from nbxmpp import JID
 
 from gajim.common import app
 from gajim.common.const import AvatarSize
-from gajim.common.const import KindConstant
+from gajim.common.const import Draft
 from gajim.common.const import RowHeaderType
-from gajim.common.helpers import AdditionalDataDict
-from gajim.common.helpers import get_group_chat_nick
 from gajim.common.helpers import get_groupchat_name
-from gajim.common.helpers import get_retraction_text
+from gajim.common.helpers import get_moderation_text
 from gajim.common.helpers import get_uf_relative_time
 from gajim.common.helpers import message_needs_highlight
 from gajim.common.i18n import _
 from gajim.common.modules.contacts import BareContact
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
+from gajim.common.modules.message_util import get_nickname_from_message
 from gajim.common.preview_helpers import filename_from_uri
 from gajim.common.preview_helpers import format_geo_coords
 from gajim.common.preview_helpers import guess_simple_file_type
 from gajim.common.preview_helpers import split_geo_uri
+from gajim.common.storage.archive import models as mod
+from gajim.common.storage.archive.const import ChatDirection
+from gajim.common.storage.archive.const import MessageType
 from gajim.common.storage.draft import DraftStorage
 from gajim.common.types import ChatContactT
 
@@ -147,64 +139,63 @@ class ChatListRow(Gtk.ListBoxRow):
             self._ui.unread_label.get_style_context().add_class(
                 'unread-counter-silent')
 
-        self._display_last_conversation_line()
+        self._display_last_conversation_row()
 
-    def _display_last_conversation_line(self) -> None:
-        line = app.storage.archive.get_last_conversation_line(
+    def _display_last_conversation_row(self) -> None:
+        message = app.storage.archive.get_last_conversation_row(
             self.contact.account, self.contact.jid)
-        if line is None:
+        if message is None:
             self.show_all()
             return
 
-        if line.message is not None:
-            message_text = line.message
+        assert isinstance(
+            self.contact,
+            BareContact | GroupchatContact | GroupchatParticipant)
 
-            if line.additional_data is not None:
-                retracted_by = line.additional_data.get_value(
-                    'retracted', 'by')
-                if retracted_by is not None:
-                    reason = line.additional_data.get_value(
-                        'retracted', 'reason')
-                    message_text = get_retraction_text(
-                        retracted_by, reason)
+        if message.text is not None:
+            message_text = message.text
 
-            me_nickname = None
-            if line.kind in (KindConstant.CHAT_MSG_SENT,
-                             KindConstant.SINGLE_MSG_SENT):
+            if message.corrections:
+                message_text = message.get_last_correction().text
+                assert message_text is not None
+
+            if message.moderation is not None:
+                message_text = get_moderation_text(
+                    message.moderation.by, message.moderation.reason)
+
+            # Nickname
+            nickname = None
+            if (message.type == MessageType.CHAT and
+                    message.direction == ChatDirection.OUTGOING):
                 self.set_nick(_('Me'))
-                me_nickname = app.nicks[self.contact.account]
+                nickname = app.nicks[self.contact.account]
 
-            if line.kind == KindConstant.GC_MSG:
-                our_nick = get_group_chat_nick(
-                    self.contact.account, self.contact.jid)
-                if line.contact_name == our_nick:
+            elif message.type in (MessageType.GROUPCHAT, MessageType.PM):
+                nickname = get_nickname_from_message(message) or None
+                if message.direction == ChatDirection.OUTGOING:
                     self.set_nick(_('Me'))
-                    me_nickname = our_nick
                 else:
-                    self.set_nick(line.contact_name)
-                    me_nickname = line.contact_name
+                    self.set_nick(nickname)
 
             self.set_message_text(
                 message_text,
-                nickname=me_nickname,
-                additional_data=line.additional_data)
+                nickname=nickname,
+                oob=message.oob)
 
-            self.set_timestamp(line.time)
+            self.set_timestamp(message.timestamp)
 
-            self.stanza_id = line.stanza_id
-            self.message_id = line.message_id
+            self.stanza_id = message.stanza_id
+            self.message_id = message.id
 
-        if line.kind in (KindConstant.FILE_TRANSFER_INCOMING,
-                         KindConstant.FILE_TRANSFER_OUTGOING):
+        if message.filetransfers:
             self.set_message_text(
                 _('File'), icon_name='text-x-generic-symbolic')
-            self.set_timestamp(line.time)
+            self.set_timestamp(message.timestamp)
 
-        if line.kind in (KindConstant.CALL_INCOMING,
-                         KindConstant.CALL_OUTGOING):
+        if message.call is not None:
             self.set_message_text(
                 _('Call'), icon_name='call-start-symbolic')
-            self.set_timestamp(line.time)
+            self.set_timestamp(message.timestamp)
 
         self.show_all()
 
@@ -260,7 +251,7 @@ class ChatListRow(Gtk.ListBoxRow):
                          text: str,
                          nickname: str | None = None,
                          icon_name: str | None = None,
-                         additional_data: AdditionalDataDict | None = None
+                         oob: list[mod.OOB] | None = None
                          ) -> None:
 
         assert isinstance(
@@ -277,8 +268,8 @@ class ChatListRow(Gtk.ListBoxRow):
         icon = None
         if icon_name is not None:
             icon = Gio.Icon.new_for_string(icon_name)
-        if additional_data is not None:
-            if app.preview_manager.is_previewable(text, additional_data):
+        if oob:
+            if app.preview_manager.is_previewable(text, oob):
                 scheme = urlparse(text).scheme
                 if scheme == 'geo':
                     location = split_geo_uri(text)
@@ -291,7 +282,7 @@ class ChatListRow(Gtk.ListBoxRow):
                     text = f'{file_type} ({file_name})'
 
         text = GLib.markup_escape_text(text)
-        if text.startswith('/me') and nickname is not None:
+        if text.startswith('/me ') and nickname is not None:
             nickname = GLib.markup_escape_text(nickname)
             text = text.replace('/me', f'* {nickname}', 1)
             text = f'<i>{text}</i>'
@@ -307,7 +298,7 @@ class ChatListRow(Gtk.ListBoxRow):
             self._ui.message_icon.set_from_gicon(icon, Gtk.IconSize.MENU)
             self._ui.message_icon.show()
 
-    def set_nick(self, nickname: str) -> None:
+    def set_nick(self, nickname: str | None) -> None:
         self._ui.nick_label.set_visible(bool(nickname))
         self._ui.nick_label.set_text(
             _('%(nickname)s:') % {'nickname': nickname})
@@ -315,11 +306,11 @@ class ChatListRow(Gtk.ListBoxRow):
     def get_real_unread_count(self) -> int:
         return self._unread_count
 
-    def set_stanza_id(self, stanza_id: str) -> None:
+    def set_stanza_id(self, stanza_id: str | None) -> None:
         self.stanza_id = stanza_id
 
-    def set_timestamp(self, timestamp: float) -> None:
-        self.timestamp = timestamp
+    def set_timestamp(self, timestamp: datetime) -> None:
+        self.timestamp = timestamp.timestamp()
         self.update_time()
 
     def update_account_identifier(self) -> None:
@@ -357,7 +348,12 @@ class ChatListRow(Gtk.ListBoxRow):
         self._ui.timestamp_label.set_text(
             get_uf_relative_time(datetime.fromtimestamp(self.timestamp)))
 
+    def update_state(self) -> None:
+        self.update_time()
+        self._ui.mute_image.set_visible(self.contact.is_muted)
+
     def add_unread(self, text: str) -> None:
+        assert self.message_id is not None
         self._unread_count += 1
         self._update_unread()
         app.storage.cache.set_unread_count(
@@ -414,7 +410,7 @@ class ChatListRow(Gtk.ListBoxRow):
                          _draft_storage: DraftStorage,
                          _signal_name: str,
                          contact: ChatContactT,
-                         draft: str
+                         draft: Draft | None,
                          ) -> None:
 
         if contact != self.contact:
@@ -422,14 +418,14 @@ class ChatListRow(Gtk.ListBoxRow):
 
         self._show_draft(draft)
 
-    def _show_draft(self, draft: str) -> None:
-        if not draft:
+    def _show_draft(self, draft: Draft | None) -> None:
+        if draft is None:
             self._ui.message_label.get_style_context().remove_class('draft')
-            self._display_last_conversation_line()
+            self._display_last_conversation_row()
             return
 
         self.set_nick('')
-        self._ui.message_label.set_text(_('Draft: %s') % draft)
+        self._ui.message_label.set_text(_('Draft: %s') % draft.text)
         self._ui.message_label.get_style_context().add_class('draft')
 
     def _on_state_flags_changed(self,
