@@ -1,25 +1,12 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
-from typing import Optional
-from typing import Union
-
 import logging
 import sys
-import time
+from functools import partial
 from urllib.parse import urlparse
 
 from gi.repository import Gdk
@@ -28,7 +15,7 @@ from gi.repository import GLib
 from gi.repository import Gtk
 from nbxmpp.errors import StanzaError
 from nbxmpp.protocol import JID
-from nbxmpp.structs import MessageProperties
+from nbxmpp.task import Task
 
 from gajim.common import app
 from gajim.common import events
@@ -42,6 +29,9 @@ from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.preview_helpers import format_geo_coords
 from gajim.common.preview_helpers import split_geo_uri
+from gajim.common.storage.archive.const import ChatDirection
+from gajim.common.storage.archive.const import MessageType
+from gajim.common.storage.archive.models import Message
 from gajim.common.structs import OutgoingMessage
 from gajim.common.types import ChatContactT
 from gajim.common.util.text import remove_invalid_xml_chars
@@ -69,7 +59,8 @@ class ChatStack(Gtk.Stack, EventHelper):
         self.set_vexpand(True)
         self.set_hexpand(True)
 
-        self._current_contact: Optional[ChatContactT] = None
+        self._current_contact: ChatContactT | None = None
+        self._last_quoted_id: int | None = None
 
         self.add_named(ChatPlaceholderBox(), 'empty')
 
@@ -155,7 +146,6 @@ class ChatStack(Gtk.Stack, EventHelper):
 
         self.register_events([
             ('message-received', 85, self._on_message_received),
-            ('gc-message-received', 85, self._on_message_received),
             ('muc-disco-update', 85, self._on_muc_disco_update),
             ('account-connected', 85, self._on_account_state),
             ('account-disconnected', 85, self._on_account_state),
@@ -170,8 +160,7 @@ class ChatStack(Gtk.Stack, EventHelper):
             self._chat_function_page.process_escape()
             return True
 
-        if self._message_action_box.is_correcting:
-            self._message_action_box.toggle_message_correction()
+        if self._message_action_box.process_escape():
             return True
 
         return False
@@ -190,11 +179,16 @@ class ChatStack(Gtk.Stack, EventHelper):
         clipboard = Gtk.Clipboard.get(Gdk.SELECTION_PRIMARY)
         old_primary_clipboard = clipboard.wait_for_text()
 
+        self._last_quoted_id = None
+
         if self._current_contact is not None:
             self._current_contact.disconnect_all_from_obj(self)
 
         client = app.get_client(account)
-        self._current_contact = client.get_module('Contacts').get_contact(jid)
+        contact = client.get_module('Contacts').get_contact(jid)
+        assert isinstance(
+            contact, BareContact | GroupchatContact | GroupchatParticipant)
+        self._current_contact = contact
 
         app.preview_manager.clear_previews()
 
@@ -227,7 +221,7 @@ class ChatStack(Gtk.Stack, EventHelper):
 
         if isinstance(self._current_contact, GroupchatContact):
             muc_data = client.get_module('MUC').get_muc_data(
-                str(self._current_contact.jid))
+                self._current_contact.jid)
             if muc_data is not None:
                 if muc_data.state.is_captcha_request:
                     self._show_chat_function_page(FunctionMode.CAPTCHA_REQUEST)
@@ -266,16 +260,14 @@ class ChatStack(Gtk.Stack, EventHelper):
 
     def _on_room_password_required(self,
                                    _contact: GroupchatContact,
-                                   _signal_name: str,
-                                   _properties: MessageProperties
+                                   _signal_name: str
                                    ) -> None:
 
         self._show_chat_function_page(FunctionMode.PASSWORD_REQUEST)
 
     def _on_room_captcha_challenge(self,
                                    contact: GroupchatContact,
-                                   _signal_name: str,
-                                   _properties: MessageProperties
+                                   _signal_name: str
                                    ) -> None:
 
         self._show_chat_function_page(FunctionMode.CAPTCHA_REQUEST)
@@ -283,39 +275,34 @@ class ChatStack(Gtk.Stack, EventHelper):
     def _on_room_captcha_error(self,
                                _contact: GroupchatContact,
                                _signal_name: str,
-                               error: StanzaError
+                               error: str
                                ) -> None:
 
-        error_text = helpers.to_user_string(error)
-        self._show_chat_function_page(FunctionMode.CAPTCHA_ERROR, error_text)
+        self._show_chat_function_page(FunctionMode.CAPTCHA_ERROR, error)
 
     def _on_room_creation_failed(self,
                                  _contact: GroupchatContact,
                                  _signal_name: str,
-                                 properties: MessageProperties
+                                 error: str
                                  ) -> None:
 
-        assert properties.error is not None
-        error_text = helpers.to_user_string(properties.error)
-        self._show_chat_function_page(FunctionMode.CREATION_FAILED, error_text)
+        self._show_chat_function_page(FunctionMode.CREATION_FAILED, error)
 
     def _on_room_join_failed(self,
                              _contact: GroupchatContact,
                              _signal_name: str,
-                             error: StanzaError
+                             error: str
                              ) -> None:
 
-        self._show_chat_function_page(
-            FunctionMode.JOIN_FAILED, helpers.to_user_string(error))
+        self._show_chat_function_page(FunctionMode.JOIN_FAILED, error)
 
     def _on_room_config_failed(self,
                                _contact: GroupchatContact,
                                _signal_name: str,
-                               error: StanzaError
+                               error: str
                                ) -> None:
 
-        self._show_chat_function_page(
-            FunctionMode.CONFIG_FAILED, helpers.to_user_string(error))
+        self._show_chat_function_page(FunctionMode.CONFIG_FAILED)
 
     def _on_muc_state_changed(self,
                               contact: GroupchatContact,
@@ -358,10 +345,10 @@ class ChatStack(Gtk.Stack, EventHelper):
 
         self._update_group_chat_actions(self._current_contact)
 
-    def _on_account_state(self,
-                          event: Union[events.AccountConnected,
-                                       events.AccountDisconnected]
-                          ) -> None:
+    def _on_account_state(
+        self,
+        event: events.AccountConnected | events.AccountDisconnected
+    ) -> None:
 
         if self._current_contact is None:
             return
@@ -379,35 +366,46 @@ class ChatStack(Gtk.Stack, EventHelper):
             self._update_chat_actions(self._current_contact)
 
     def _on_message_received(self, event: events.MessageReceived) -> None:
-        if not event.msgtxt or event.properties.is_sent_carbon:
+        if event.from_mam:
             return
 
-        client = app.get_client(event.account)
-        contact = client.get_module('Contacts').get_contact(event.jid)
-        if isinstance(contact, GroupchatContact):
-            # MUC messages may be received after some delay, so make sure we
-            # don't issue notifications for our own messages.
-            self_contact = contact.get_self()
-            if (self_contact is not None and
-                    self_contact.name == event.properties.muc_nickname):
-                return
-
         if app.window.is_chat_active(event.account, event.jid):
             return
 
-        self._issue_notification(event)
+        message = event.message
+        if message.direction == ChatDirection.OUTGOING:
+            return
 
-    def _issue_notification(self, event: events.MessageReceived) -> None:
-        text = event.msgtxt
-        tim = event.properties.timestamp
-        additional_data = event.additional_data
-        client = app.get_client(event.account)
-        contact = client.get_module('Contacts').get_contact(event.jid)
+        if event.m_type == MessageType.GROUPCHAT:
+            client = app.get_client(event.account)
+            contact = client.get_module('Contacts').get_contact(
+                event.jid, groupchat=True)
+            assert isinstance(contact, GroupchatContact)
+            # MUC messages may be received after some delay, so make sure we
+            # don't issue notifications for our own messages.
+            self_contact = contact.get_self()
+            if (self_contact is not None and
+                    self_contact.name == message.resource):
+                return
+
+        self._issue_notification(event.account, message)
+
+    def _issue_notification(
+        self,
+        account: str,
+        data: Message
+    ) -> None:
+
+        text = data.text
+        assert text is not None
+
+        client = app.get_client(account)
+        contact = client.get_module('Contacts').get_contact(data.remote.jid)
 
         title = _('New message from')
 
         is_previewable = app.preview_manager.is_previewable(
-            text, additional_data)
+            text, data.oob)
         if is_previewable:
             scheme = urlparse(text).scheme
             if scheme == 'geo':
@@ -419,7 +417,7 @@ class ChatStack(Gtk.Stack, EventHelper):
                 _icon, file_type = preview_helpers.guess_simple_file_type(text)
                 text = f'{file_type} ({file_name})'
 
-        sound: Optional[str] = None
+        sound: str | None = None
         msg_type = 'chat-message'
         if isinstance(contact, BareContact):
             msg_type = 'chat-message'
@@ -429,7 +427,7 @@ class ChatStack(Gtk.Stack, EventHelper):
 
         if isinstance(contact, GroupchatContact):
             msg_type = 'group-chat-message'
-            title += f' {event.resource} ({contact.name})'
+            title += f' {data.resource} ({contact.name})'
             assert contact.nickname is not None
             needs_highlight = helpers.message_needs_highlight(
                 text, contact.nickname, client.get_own_jid().bare)
@@ -441,23 +439,20 @@ class ChatStack(Gtk.Stack, EventHelper):
             if not contact.can_notify() and not needs_highlight:
                 return
 
-            if contact.can_notify() or needs_highlight:
-                set_urgency_hint(app.window, True)
+            set_urgency_hint(app.window, True)
 
         if isinstance(contact, GroupchatParticipant):
             msg_type = 'private-chat-message'
             title += f' {contact.name} (private in {contact.room.name})'
             sound = 'first_message_received'
 
-        # Is it a history message? Don't want sound-floods when we join.
-        if tim is not None and time.mktime(time.localtime()) - tim > 1:
-            sound = None
-
+        assert isinstance(
+            contact, BareContact | GroupchatContact | GroupchatParticipant)
         if app.settings.get('notification_preview_message'):
-            if text.startswith('/me') or text.startswith('/me\n'):
+            if text.startswith('/me '):
                 name = contact.name
                 if isinstance(contact, GroupchatContact):
-                    name = event.properties.muc_nickname
+                    name = data.resource
                 text = f'* {name} {text[3:]}'
         else:
             text = ''
@@ -490,6 +485,8 @@ class ChatStack(Gtk.Stack, EventHelper):
             'muc-change-role',
             'muc-change-affiliation',
             'muc-request-voice',
+            'quote-next',
+            'quote-prev',
         ]
 
         for action in actions:
@@ -516,7 +513,6 @@ class ChatStack(Gtk.Stack, EventHelper):
             jingle.get_enabled() or
             httpupload.get_enabled())
 
-        app.window.get_action('show-contact-info').set_enabled(online)
         app.window.get_action('correct-message').set_enabled(online)
 
     def _update_chat_actions(self, contact: BareContact) -> None:
@@ -524,14 +520,15 @@ class ChatStack(Gtk.Stack, EventHelper):
         online = app.account_is_connected(account)
 
         app.window.get_action('start-voice-call').set_enabled(
-            online and contact.supports_audio() and
+            online and contact.supports_audio and
             sys.platform != 'win32')
         app.window.get_action('start-video-call').set_enabled(
-            online and contact.supports_video() and
+            online and contact.supports_video and
             sys.platform != 'win32')
 
         app.window.get_action('quote').set_enabled(online)
         app.window.get_action('mention').set_enabled(online)
+        app.window.get_action('reply').set_enabled(online)
 
     def _update_group_chat_actions(self, contact: GroupchatContact) -> None:
         joined = contact.is_joined
@@ -541,9 +538,7 @@ class ChatStack(Gtk.Stack, EventHelper):
             assert self_contact
             is_visitor = self_contact.role.is_visitor
 
-        app.window.get_action('muc-change-nickname').set_enabled(
-            joined and not contact.is_irc)
-
+        app.window.get_action('muc-change-nickname').set_enabled(joined)
         app.window.get_action('muc-contact-info').set_enabled(joined)
         app.window.get_action('muc-execute-command').set_enabled(joined)
         app.window.get_action('muc-ban').set_enabled(joined)
@@ -556,7 +551,8 @@ class ChatStack(Gtk.Stack, EventHelper):
 
         app.window.get_action('quote').set_enabled(joined)
         app.window.get_action('mention').set_enabled(joined)
-        app.window.get_action('retract-message').set_enabled(joined)
+        app.window.get_action('reply').set_enabled(joined)
+        app.window.get_action('moderate-message').set_enabled(joined)
 
     def _update_participant_actions(self,
                                     contact: GroupchatParticipant) -> None:
@@ -564,11 +560,16 @@ class ChatStack(Gtk.Stack, EventHelper):
 
     def _on_action(self,
                    action: Gio.SimpleAction,
-                   param: Optional[GLib.Variant]) -> None:
+                   param: GLib.Variant | None) -> None:
+
+        if self.get_visible_child_name() != 'controls':
+            return
 
         action_name = action.get_name()
         contact = self._current_contact
-        assert contact is not None
+        if contact is None:
+            return
+
         account = contact.account
         client = app.get_client(account)
         jid = contact.jid
@@ -631,7 +632,7 @@ class ChatStack(Gtk.Stack, EventHelper):
                 assert isinstance(contact, GroupchatContact)
                 resource_contact = contact.get_resource(nick)
                 jid = resource_contact.jid
-            open_window('AdHocCommands', account=account, jid=jid)
+            open_window('AdHocCommands', account=account, jids=[jid])
 
         elif action_name == 'muc-kick':
             assert param is not None
@@ -645,19 +646,74 @@ class ChatStack(Gtk.Stack, EventHelper):
 
         elif action_name == 'muc-change-role':
             assert param is not None
+            assert isinstance(contact, GroupchatContact)
             nick, role = param.get_strv()
-            client.get_module('MUC').set_role(contact.jid, nick, role)
+            client.get_module('MUC').set_role(
+                contact.jid,
+                nick,
+                role,
+                callback=partial(
+                    self._on_affiliation_or_role_change,
+                    contact, jid, role),
+            )
 
         elif action_name == 'muc-change-affiliation':
             assert param is not None
+            assert isinstance(contact, GroupchatContact)
             jid, affiliation = param.get_strv()
             client.get_module('MUC').set_affiliation(
                 contact.jid,
-                {jid: {'affiliation': affiliation}})
+                {jid: {'affiliation': affiliation}},
+                callback=partial(
+                    self._on_affiliation_or_role_change,
+                    contact, jid, affiliation),
+            )
 
         elif action_name == 'muc-request-voice':
             client.get_module('MUC').request_voice(contact.jid)
 
+        elif action_name.startswith('quote-'):
+            view = self._chat_control.get_conversation_view()
+            if action_name == 'quote-prev':
+                row = view.get_prev_message_row(self._last_quoted_id)
+            else:
+                row = view.get_next_message_row(self._last_quoted_id)
+
+            if row is not None:
+                self._last_quoted_id = row.pk
+                self._message_action_box.insert_as_quote(
+                    row.get_text(), clear=True)
+
+    def _on_affiliation_or_role_change(self,
+                                       muc: GroupchatContact,
+                                       jid: JID | str,
+                                       affiliation_or_role: str,
+                                       task: Task) -> None:
+        try:
+            result = task.finish()
+        except StanzaError as error:
+            log.error('Error on affiliation/role change request: %s',
+                      error)
+            if self._current_contact == muc:
+                self._chat_control.add_info_message(
+                    _('An error occurred while trying to make '
+                      '{occupant_jid} {affiliation_or_role}: {error}').format(
+                        occupant_jid=jid,
+                        affiliation_or_role=affiliation_or_role,
+                        error=str(error),
+                    ))
+            else:
+                ErrorDialog(
+                    _('An error occurred while trying to make '
+                      '{occupant_jid} {affiliation_or_role} in group '
+                      '"{group}"').format(
+                        occupant_jid=jid,
+                        affiliation_or_role=affiliation_or_role,
+                        group=muc.name),
+                    str(error))
+        else:
+            log.debug('Affiliation/role change success: %s', result)
+
     def _on_drag_data_received(self,
                                _widget: Gtk.Widget,
                                _context: Gdk.DragContext,
@@ -687,7 +743,7 @@ class ChatStack(Gtk.Stack, EventHelper):
 
     def _on_drag_motion(self,
                         _widget: Gtk.Widget,
-                        _context: Gdk.DragContext,
+                        context: Gdk.DragContext,
                         _x_coord: int,
                         _y_coord: int,
                         time: int
@@ -697,14 +753,18 @@ class ChatStack(Gtk.Stack, EventHelper):
             # Workaround for https://gitlab.gnome.org/GNOME/gtk/-/issues/5518
             return False
 
-        self._drop_area.set_no_show_all(False)
-        self._drop_area.show_all()
-        return True
+        for targ in context.list_targets():
+            if targ.name() in {'text/uri-list', 'application/vnd.portal.files'}:
+                self._drop_area.set_no_show_all(False)
+                self._drop_area.show_all()
+                return True
+
+        return False
 
     def _show_chat_function_page(self,
                                  function_mode: FunctionMode,
-                                 data: Optional[str] = None,
-                                 files: Optional[list[str]] = None
+                                 data: str | None = None,
+                                 files: list[str] | None = None
                                  ) -> None:
 
         assert self._current_contact is not None
@@ -734,7 +794,7 @@ class ChatStack(Gtk.Stack, EventHelper):
         self._chat_control.add_info_message(message)
 
     def _on_send_message(self) -> None:
-        message = self._message_action_box.msg_textview.get_text()
+        message = self._message_action_box.get_text()
         if message.startswith('//'):
             # Escape sequence for chat commands
             message = message[1:]
@@ -742,8 +802,14 @@ class ChatStack(Gtk.Stack, EventHelper):
         contact = self._current_contact
         assert contact is not None
 
+        client = app.get_client(contact.account)
+
         encryption = contact.settings.get('encryption')
-        if encryption:
+        if encryption == 'OMEMO':
+            if not client.get_module('OMEMO').check_send_preconditions(contact):
+                return
+
+        elif encryption:
             if encryption not in app.plugin_manager.encryption_plugins:
                 ErrorDialog(_('Encryption error'),
                             _('Missing necessary encryption plugin'))
@@ -756,43 +822,33 @@ class ChatStack(Gtk.Stack, EventHelper):
             if not self._chat_control.sendmessage:
                 return
 
-        client = app.get_client(contact.account)
-
         message = remove_invalid_xml_chars(message)
         if message in ('', '\n'):
             return
 
         label = self._message_action_box.get_seclabel()
-
-        correct_id = None
-        if self._message_action_box.is_correcting:
-            correct_id = self._message_action_box.try_message_correction(
-                message)
-            if correct_id is None:
-                return
+        correct_id = self._message_action_box.get_correction_id()
+        reply_data = self._message_action_box.get_message_reply()
 
         chatstate = client.get_module('Chatstate').get_active_chatstate(
             contact)
 
-        type_ = 'chat'
-        if isinstance(contact, GroupchatContact):
-            type_ = 'groupchat'
-
         message_ = OutgoingMessage(account=contact.account,
                                    contact=contact,
-                                   message=message,
-                                   type_=type_,
+                                   text=message,
                                    chatstate=chatstate,
-                                   label=label,
+                                   sec_label=label,
                                    control=self._chat_control,
-                                   correct_id=correct_id)
+                                   correct_id=correct_id,
+                                   reply_data=reply_data)
 
         client.send_message(message_)
 
-        self._message_action_box.msg_textview.clear()
-        app.storage.drafts.set(contact, '')
+        self._message_action_box.reset_state_after_send()
 
-    def get_last_message_id(self, contact: ChatContactT) -> Optional[str]:
+        self._last_quoted_id = None
+
+    def get_last_message_id(self, contact: ChatContactT) -> str | None:
         return self._message_action_box.get_last_message_id(contact)
 
     def _close_control(self) -> None:
@@ -807,6 +863,7 @@ class ChatStack(Gtk.Stack, EventHelper):
         if self._current_contact is not None:
             self._current_contact.disconnect_all_from_obj(self)
 
+        self._last_quoted_id = None
         self.set_visible_child_name('empty')
         self._chat_banner.clear()
         self._message_action_box.clear()
@@ -827,14 +884,9 @@ class ChatPlaceholderBox(Gtk.Box):
         Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL,
                          spacing=18)
         self.set_valign(Gtk.Align.CENTER)
-        pixbuf = Gtk.IconTheme.load_icon_for_scale(
-            Gtk.IconTheme.get_default(),
-            'org.gajim.Gajim-symbolic',
-            100,
-            self.get_scale_factor(),
-            Gtk.IconLookupFlags.FORCE_SIZE)
-        image = Gtk.Image.new_from_pixbuf(pixbuf)
-        image.get_style_context().add_class('dim-label')
+        image = Gtk.Image.new_from_icon_name(
+            'org.gajim.Gajim-symbolic', Gtk.IconSize.from_name('100'))
+        image.set_opacity(0.2)
         self.add(image)
 
         button = Gtk.Button(label=_('Start Chatting…'))
