@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -25,6 +15,7 @@ from gi.repository import GObject
 from gi.repository import Gtk
 from nbxmpp.errors import StanzaError
 from nbxmpp.modules.vcard4 import VCard
+from nbxmpp.namespaces import Namespace
 from nbxmpp.structs import AnnotationNote
 from nbxmpp.structs import SoftwareVersionResult
 from nbxmpp.task import Task
@@ -221,7 +212,9 @@ class ContactInfo(Gtk.ApplicationWindow, EventHelper):
         if note is not None:
             self._ui.textview_annotation.get_buffer().set_text(note.data)
 
-        if app.account_supports_private_storage(self.account):
+        server_disco = self._client.get_module('Discovery').server_info
+        if (server_disco is not None and
+                server_disco.supports(Namespace.PRIVATE)):
             # Hide the Notes page if private storage is not available, because
             # roster notes cannot be stored without.
             # Since there is no disco mechanism for private storage, we rely on
