@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -125,7 +115,7 @@ class Preferences(Gtk.ApplicationWindow):
 
     def _add_video_preview(self) -> None:
         self._video_preview = VideoPreview()
-        self._ui.video.add(self._video_preview.widget)
+        self._ui.video.add(self._video_preview)
 
     def _on_key_press(self, _widget: Gtk.Widget, event: Gdk.EventKey) -> None:
         if event.keyval == Gdk.KEY_Escape:
@@ -262,6 +252,11 @@ class General(PreferenceBox):
                     SettingType.CONFIG,
                     'show_send_message_button'),
 
+            Setting(SettingKind.SWITCH,
+                    _('Show Voice Message Button'),
+                    SettingType.CONFIG,
+                    'show_voice_message_button'),
+
             Setting(SettingKind.SWITCH,
                     _('Send Messages with Control+Enter'),
                     SettingType.CONFIG,
@@ -419,7 +414,7 @@ class FilePreview(PreferenceBox):
                     'preview_size',
                     desc=_('Size of preview images in pixels'),
                     bind='enable_file_preview',
-                    props={'range_': (100, 1000)}),
+                    props={'range_': (100, 1000, 1)}),
 
             Setting(SettingKind.POPOVER,
                     _('File Size Limit'),
@@ -589,7 +584,7 @@ class AutoAwayDialog(SettingsDialog):
                     SettingType.CONFIG,
                     'autoawaytime',
                     desc=_('Minutes until your status gets changed'),
-                    props={'range_': (1, 720)},
+                    props={'range_': (1, 720, 1)},
                     bind='autoaway'),
 
             Setting(SettingKind.ENTRY,
@@ -617,7 +612,7 @@ class AutoExtendedAwayDialog(SettingsDialog):
                     SettingType.CONFIG,
                     'autoxatime',
                     desc=_('Minutes until your status gets changed'),
-                    props={'range_': (1, 720)},
+                    props={'range_': (1, 720, 1)},
                     bind='autoxa'),
 
             Setting(SettingKind.ENTRY,
@@ -643,6 +638,13 @@ class Themes(PreferenceBox):
         }
 
         settings = [
+            Setting(SettingKind.SPIN,
+                    _('User Interface Font Size'),
+                    SettingType.CONFIG,
+                    'app_font_size',
+                    props={'range_': (1.0, 1.5, 0.125)},
+                    callback=self._on_app_font_size_changed),
+
             Setting(SettingKind.POPOVER,
                     _('Dark Theme'),
                     SettingType.CONFIG,
@@ -663,11 +665,14 @@ class Themes(PreferenceBox):
 
         PreferenceBox.__init__(self, settings)
 
+    @staticmethod
+    def _on_app_font_size_changed(_value: float, *args: Any) -> None:
+        app.css_config.apply_app_font_size()
+
     @staticmethod
     def _get_theme_items() -> list[str]:
         theme_items = ['default']
-        for settings_theme in app.css_config.themes:
-            theme_items.append(settings_theme)
+        theme_items.extend(app.css_config.themes)
         return theme_items
 
     def update_theme_list(self) -> None:
@@ -729,7 +734,7 @@ class StunServerDialog(SettingsDialog):
 class Audio(PreferenceBox):
     def __init__(self, *args: Any) -> None:
 
-        deps_installed = app.is_installed('AV')
+        deps_installed = app.is_installed('GST')
 
         audio_input_devices = {}
         audio_output_devices = {}
