@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -287,7 +277,7 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
                             _action: Gio.SimpleAction,
                             param: GLib.Variant) -> None:
 
-        app.window.execute_command(self._account, param.get_string())
+        app.window.execute_command(self._account, param.get_strv())
 
     def _on_block_contact(self,
                           _action: Gio.SimpleAction,
@@ -392,7 +382,8 @@ class Roster(Gtk.ScrolledWindow, EventHelper):
         gateway_register = contact.is_gateway and contact.supports(
             Namespace.REGISTER)
 
-        menu = get_roster_menu(self._account, jid, gateway=gateway_register)
+        menu = get_roster_menu(
+            self._account, contact, gateway=gateway_register)
         popover = GajimPopover(menu, relative_to=treeview, event=event)
         popover.popup()
 
